import React, { useState, useEffect } from 'react'
import { Input, message, Pagination } from 'antd'
import { SnippetsOutlined } from '@ant-design/icons'
import { GetAccountConfigInfo, GetAccountPageList, getStationListByUserID } from '../../../../apis/process'
import styles from './index.less';
import { isJson } from '../../../../utils'
import TablePack from '../../../../core/Account/components/TablePack'
import SearchGroup from '../../../../core/Account/components/SearchGroup'
import Drag from '../../../components/Drag'

const AccountSelector = (props) => {

  const userID = window?.globalConfig?.userInfo?.OID || 1
  const { value, onChange, schema, addons } = props
  const { disabled, accountName, fieldshine, siteFilter, sql, isMultiple, presetValue, placeholder } = schema

  const [params, setParams] = useState({
    userID: userID,
    accountName,
    sortFields: '录入时间',
    direction: 'desc',
    total: 0,
    pageIndex: 1,
    pageSize: 100
  })
  const [loading, setLoading] = useState(false)
  const [dataSource, setDataSource] = useState([])
  const [config, setConfig] = useState({ webShowFieldGroup: '' })
  const [visible, setVisible] = useState(false)
  const [keys, setKeys] = useState([])

  const iconClick = () => {
    if (addons) {
      if (!accountName) {
        return message.info('请配置台账名称！');
      }
      if (!fieldshine.length) {
        return message.info('请配置映射字段！');
      }
      setVisible(true);
      if (!config.webShowFieldGroup) {
        getConfig();
      }
      getDataSource();
    }
  }

  const getConfig = async () => {
    const { code, data } = await GetAccountConfigInfo(accountName);
    if (code == 0) {
      setConfig({ ...data, formJson: isJson(data.formJson) ? JSON.parse(data.formJson) : {} })
    }
  }

  const getDataSource = async (outParams = {}) => {
    let queryWheres = [];
    if (siteFilter) {
      const { code, data } = await getStationListByUserID(...[userID, false]);
      if (code === 0) {
        queryWheres = [
          {
            field: '所属站点',
            type: '包括',
            value: data.join(',')
          }
        ];
      }
    }
    setLoading(true);
    let param = {
      userID: userID,
      accountName: accountName,
      condition: sql ? window.btoa(encodeURIComponent(sql)) : '',
      sortFields: '录入时间',
      queryWheres: queryWheres,
      ...params,
      ...outParams,
    };
    const { data, code, msg } = await GetAccountPageList(param);
    if (code === 0) {
      if (data.jsonData) {
        let dataSource = JSON.parse(data.jsonData);
        setDataSource(dataSource);
        let valueArr = value ? value.split(',') : [];
        let rows = dataSource.filter(v => valueArr.includes(v[fieldshine[0].fromField]));
        setKeys(rows.map(v => v.ID));
        setParams({ ...params, ...outParams, total: data.totalCount });
      }
    } else {
      message.error(msg);
    }
    setLoading(false);
  }

  const tableChange = (page, filters, sorter) => {
    let queryWheres = []
    Object.keys(filters).forEach(k => {
      if (filters[k]?.[0]) {
        queryWheres.push({ field: k, type: '包括', value: filters[k][0] })
      }
    })
    let param = {
      sortFields: sorter.order ? sorter.field : '录入时间',
      direction: sorter.order !== 'ascend' ? 'desc' : 'asc',
      queryWheres
    }
    setParams({ ...params, ...param })
    getDataSource(param)
  }

  const rowClick = (row) => {
    if (isMultiple) {
      if (keys.includes(row.ID)) {
        let values = keys.filter(v => v !== row.ID)
        setKeys(values)
      } else {
        let values = [...keys, row.ID]
        setKeys(values)
      }
    } else {
      setKeys([row.ID]);
    }
  }

  const onOk = () => {
    if (fieldshine.length > 1) {
      let row = dataSource.find(v => v.ID === keys[0]) || {};
      Object.keys(addons.formData).forEach(child => {
        Object.keys(addons.formData[child]).forEach(sun => {
          fieldshine.forEach(v => {
            if (sun === v.toField) {
              const schema = addons.getSchemaByPath(`${child}.${sun}`);
              if (schema.widget === 'fileUpload') {
                addons.setSchemaByPath(`${child}.${sun}`, { ...schema, otherFile: row[v.fromField] });
              } else {
                addons.setValue(`${child}.${sun}`, row[v.fromField] || '');
              }
            }
          });
        });
      });
    } else {
      let array = dataSource.filter(v => keys.includes(v.ID));
      if (array.length) {
        let rowValues = array.map(v => v[fieldshine[0].fromField]).filter(v => v).join(',');
        onChange(rowValues);
      }
    }
    setVisible(false);
  }

  const search = (values) => {
    setParams({ ...params, ...values })
    getDataSource({ ...values })
  }

  const pageChange = (pageIndex, pageSize) => {
    getDataSource({ pageIndex, pageSize })
  }

  useEffect(() => {
    if (addons) {
      addons.setValue(addons.dataPath, presetValue);
    } else {
      onChange(presetValue);
    }
  }, [presetValue])

  const addonAfter = <SnippetsOutlined onClick={() => iconClick()} style={{ color: 'rgba(0, 0, 0, 0.25)' }} />

  return (
    <div className={styles.AccountSelector}>
      <Input
        placeholder={!disabled ? placeholder : null}
        value={value}
        onClick={() => iconClick()}
        addonAfter={!disabled ? addonAfter : null}
        disabled={disabled}
      />
      <Drag
        onCancel={() => setVisible(false)}
        onOk={onOk}
        width='60%'
        title={accountName}
        visible={visible}
        getContainer={false}
        bodyStyle={{ height: 570, paddingBottom: 0 }}
      >
        <div className={styles.content}>
          <div className={styles.top}>
            <SearchGroup
              onChange={search}
              readOnly={true}
            />
          </div>
          <div className={styles.bottom}>
            <TablePack
              loading={loading}
              readOnly={true}
              config={config}
              dataSource={dataSource}
              btnsClick={() => { }}
              tableChange={tableChange}
              parent='台账选择器'
              rowSelection={{
                type: isMultiple && fieldshine.length === 1 ? 'checkbox' : 'radio',
                selectedRowKeys: keys,
                fixed: 'left',
                onChange: (keys) => setKeys(keys)
              }}
              onRow={record => ({
                onClick: event => {
                  event.stopPropagation();
                  rowClick(record);
                },
              })}
            />
          </div>
          <div className={styles.footer}>
            <div className={styles.total}>共计{params.total}条数据</div>
            <div className={styles.pagination}>
              <Pagination
                size='small'
                showQuickJumper
                pageSize={params.pageSize}
                current={params.pageIndex}
                total={params.total}
                onChange={pageChange}
              />
            </div>
          </div>
        </div>
      </Drag>
    </div>
  )

}

export default AccountSelector